package meerkat.mixer.mixing;

import java.util.ArrayList;

/**
 * A generic permutation network composed of 2-input switches.
 */
public interface PermutationNetwork {
    /**
     * Return the number of inputs supported by the network.
     * @return
     */
    int getNumInputs();

    /**
     * Return the number of layers supported by the network.
     * @return
     */
    int getNumLayers();

    int getOutputIdxInPreviousLayer(int layer, int inputIdx);

    /**
     * Initialize switches to generate a specific permutation.
     * @param permutation
     */
    void setPermutation(int[] permutation);


    /**
     * Apply a single layer's permutation (as implied by {@link #setPermutation(int[])})
     * @param values values output by previous layer
     * @param layer layer to apply (layer is between 1 and numLayers; layer 0 is the input layer)
     * @param <T> type of values.
     */
    <T> void permuteLayer(T[] values, int layer);

    <T> void permuteLayer(ArrayList<T> values, int layer);
}
